@@ -621,9 +621,19 @@ result in the death of your install.
 Probably the simplest fix is to tarball your .git directory
 and add the tarball to your manifest:
 
-    $ find .git | sort -d | cpio -ov -Htar > local_repo.tar;
+=over 4
 
-add "local_repo.tar" to your MANIFEST and then 
+=item Makefile.PL
+
+Add a PREOP to the WriteMakefile arguments:
+
+    dist =>
+    {
+        PREOP =>
+        q{ /bin/tar cfv local_repo.tar .git },
+    },
+
+Specify "git" and a tarball to your distro:
 
     use Module::FromPerlVer
     qw
@@ -631,13 +641,22 @@ add "local_repo.tar" to your MANIFEST and then
         use_git     1
         git_tarball local_repo.tar
     );
+    
+=item MANIFEST
 
-call the tarball whatever you like (there is no default for
-it, yet).
+Add a corresponding record to your MANIFEST:
 
-At that point the git extractor will extract the tarball,
-check for a ./.git directory, croak if there isn't one, 
-and check out the tags.
+    CHANGES
+    README
+    Makefile.PL
+    lib/Foo/Bar.pm
+
+  * local_repo.tar
+
+=back
+
+At that point "make dist" will generate an appriate tarball and
+"perl Makefile.PL" will use it as necessary.
 
 =item Git clone via URL
 
@@ -972,6 +991,14 @@ Perl versions.
 
 Describes how the files are copied by Dir.pm.
 
+=item ExtUtils::MakeMaker
+
+See the sections for "make tardist" and the tutorial.
+
+Definition of "PREOP" which can be included with the
+Makefile.PL to tarball .git into a stock path included
+into your MANIFEST.
+
 =back
 
 =head1 LICENSE
@@ -981,6 +1008,7 @@ later released version of Perl.
 
 =head1 COPYRIGHT
 
+
 Copyright 2018, Steven Lembark, all rights reserved.
 
 =head1 AUTHOR
