@@ -10,11 +10,12 @@ use parent  qw( Module::FromPerlVer::Extract );
 
 use Archive::Tar;
 
-use Cwd             qw( getcwd          );
-use File::Basename  qw( basename        );
-use FindBin         qw( $Bin            );
-use List::Util      qw( first           );
-use Symbol          qw( qualify_to_ref  );
+use Cwd                     qw( getcwd          );
+use File::Basename          qw( basename        );
+use File::Copy::Recursive   qw( dircopy         );
+use FindBin                 qw( $Bin            );
+use List::Util              qw( first           );
+use Symbol                  qw( qualify_to_ref  );
 
 ########################################################################
 # package variables & sanity checks
@@ -65,81 +66,87 @@ my $locate_tarball
     return
 };
 
-my $extract_tarball
-= sub
-{
+my @handlerz = 
+(
+    # simplest case first, to more complicated.
 
-    my $extract = shift;
-    my $tarball = shift
-    or die "Bogus extract_tarball: false tarball basename.\n";
-
-    my $path    = $extract->$locate_tarball( $tarball )
-    or die "Missing: '$tarball' ($Bin)";
-
-    print "# Extract repo from: '$path'";
-
-    for my $tar ( Archive::Tar->new )
+    sub
     {
-        $tar->extract_archive( $path )
-        and last;
+        -e './.git'
+    },
 
-        my $error   = $tar->error;
-
-        die "Failed extract: '$path'.\n$error\n"
-    };
-
-    # at this point the .git tarball should have been
-    # extracted.
-
-    -e '.git'
-    or die "Botched extract_tarball: './.git' in '$path'.\n";
-
-    '.'
-};
-
-my $clone_url
-= sub
-{
-    my $extract = shift;
-    my $url     = $extract->value( 'git_repo_url' )
-    or die "Botched clone_url: no 'git_repo_url'.\n";
-    my $cmd     = join ' ' => @clone, $url;
-    my $source  = basename $url, '.git';
-
-    print STDERR "\n# Cloning from: '$url'\n";
-
-    my $output  = qx{ $cmd };
-
-    if( my $status  = $? )
+    sub
     {
-        die <<END
+        my $extract = shift;
+        my $tball   = $extract->value( 'git_tarball' )
+        or return;
 
-Non-zero:   '$status'.
-Executing:  '$cmd'.
-Cloning:    '$url'."
-Output:
+        my $path    = $extract->$locate_tarball( $tball )
+        or die "Missing: '$tball' ($Bin)";
 
-$output
+        print "# Extract repo from: '$path'";
+
+        for my $tar ( Archive::Tar->new )
+        {
+            $tar->extract_archive( $path )
+            and last;
+
+            my $error   = $tar->error;
+
+            die "Failed extract: '$path'.\n$error\n"
+        };
+
+        # at this point the .git tarball should have been
+        # extracted.
+
+        -e '.git'
+        or die "Botched extract_tarball: no './.git' in '$path'.\n";
+    },
+
+    sub
+    {
+        my $extract = shift;
+        my $url     = $extract->value( 'git_repo_url' )
+        or return;
+        my $cmd     = join ' ' => @clone, $url;
+
+        print STDERR "\n# Cloning from: '$url'\n";
+
+        my $output  = qx{ $cmd };
+
+        if( my $status  = $? )
+        {
+            die <<END
+
+    Non-zero:   '$status'.
+    Executing:  '$cmd'.
+    Cloning:    '$url'."
+    Output:
+
+    $output
 
 END
-    }
-    else
-    {
-        my $base    = basename $url, '.git';
+        }
+        else
+        {
+            my $base    = basename $url, '.git';
+            my $git     = "$base/.git";
 
-        print STDERR "\n# Clone complete: '$base'\n";
+            print STDERR "\n# Clone complete: '$base'\n";
+            
+            -e $git
+            or die "Botched clone: missing '$git' ($url).\n";
 
-        $base
-    }
+            dircopy $git, '.';
+        }
 
-    # caller gets back the git repository path or death.
-};
+        # belt-and-suspenders: last chance to catch dircopy
+        # failing with a specific message.
 
-my $copy_source
-= sub
-{
-    my $extract = shift;
-};
+        -e '.git'
+        or die "Botched clone_url: no './.git'.\n";
+    },
+);
 
 ########################################################################
 # methods
@@ -157,31 +164,22 @@ sub module_sources
     my $extract = shift;
     my $dir     = getcwd;
 
-    if( -e '.git' )
-    {
-        # nothing more do to
-    }
-    elsif( my $tb   = $extract->value( 'git_tarball' ) )
-    {
-        $extract->$extract_tarball( $tb )
-    }
-    elsif( my $url  = $extract->value( 'git_url' ) )
-    {
-        $extract->$clone_url( $url )
-    }
-    else
-    {
-        die
-        "Botched module_sources: no git_tarball, git_url, or ./git.\n";
-    }
+    # whichever one returns indicates that there is a 
+    # working .git directory.
 
-    # at this point we need a ./.git directory.
+    first
+    {
+        $extract->$_
+    }
+    @handlerz
+    or
+    die "Botched module_sources: no tarball, URL, or local .git.\n";
 
-    -e '.git'
-    or die "Botched module_sources: no '$dir/.git'.\n";
+    # at this point expect a ./.git directory.
 
     my $prefix  = $extract->value( 'git_prefix' )
-    or die "Botched module_sources: no 'git_prefix' available.\n";
+    or
+    die "Botched module_sources: no 'git_prefix' available.\n";
     
     # force a list context for qx to get tags as array.
 
@@ -194,6 +192,9 @@ sub module_sources
 
     print join "\n#\t" => "# Git tags: '$prefix*'", @tagz;
 
+    # at this point git should be able to checkout the
+    # tag contents so get_files should work.
+
     @tagz
 }
 
@@ -220,23 +221,19 @@ sub get_files
     # no telling what the tag might look like.
     # quotes protect it from the shell.
 
-    my @status
-    = do
-    {
-        local $,    = ' ';
+    my $cmd     = join ' ' => @checkout, $tag;
 
-        qx{ @checkout '$tag' }
-    };
+    my @status = qx{ $cmd };
 
     if( $? )
     {
-        local $, = "\n@\t";
+        local $, = "\n#\t";
 
         chomp @status;
 
         print 'Checkout result:', @status;
 
-        warn "Non-zero exit: $? from @checkout '$tag'";
+        die "Non-zero exit: $? from '$cmd'.\n";
     }
     else
     {
